#############################
# filename: OpenSees2YAML.tcl
# author: Hanlin Dong
# create: 2019-05-07 14:48:46
# License: MIT License (https://opensource.org/licenses/MIT)
# (The latest version can be found on http://www.hanlindong.com/)
# description:
#     This script logs the opensees tcl model into a yaml file with the schema shown below. It works with 3D model only.
# version: 1.0
# update: 2019-05-08 13:23:35
#############################

##### INPUT ARGS #####
# maxEigen: the number of eigen values to be solved. If it is set to 0, no eigen value analysis will be conducted.
##### SCHEMA #####
# <2D>
# nodes:
#   1:
#     coords: [1,1]
#     mass: [1,1,1]
#     eigen: 
#         - [1.1, 1.1, 1.1]
#         - [2.2, 2.2, 2.2]
# elements:
#   1:
#     connection: [1, 2]
#   2:
#     connection: [1, 2, 3]
#   3:
#     connection: [1, 2, 3, 4]
#
# <3D>
# nodes:
#   1:
#     coords: [1,1,1]
#     mass: [1,1,1,1,1,1]
#     eigen: 
#         - [1.1, 1.1, 1.1, 1.1, 1.1, 1.1]
#         - [2.2, 2.2, 2.2, 2.2, 2.2, 2.2]
# elements:
#   1:
#     connection: [1, 2]

##### LOADING MESSAGE #####
puts "OpenSees2YAML version 1.0 successfully loaded ... (Author: Hanlin Dong. Licnese: MIT)"

################
#      2D      #
################
proc OpenSees2YAML2D {maxEigen} {
# eigen value solver
if {$maxEigen > 0} {
    puts "Eigen values:"
    puts [eigen $maxEigen]
} else {
    puts "Eigen value is not solved."
}
# get node tags
set nodeTags [getNodeTags]
foreach nodeTag $nodeTags {
    # node coordinates
    set coordinates($nodeTag) [format "\[%f, %f\]" [nodeCoord $nodeTag 1] [nodeCoord $nodeTag 2]]
    # node mass
    set masses($nodeTag) [format "\[%f, %f, %f\]" [nodeMass $nodeTag 1] [nodeMass $nodeTag 2] [nodeMass $nodeTag 3]]
    # node eigen
    set eigenYaml($nodeTag) {}
    if {$maxEigen > 0} {
        for {set eigen 1} {$eigen <= $maxEigen} {incr eigen} {
            lappend eigenYaml($nodeTag) [format "\[%f, %f, %f\]" [nodeEigenvector $nodeTag $eigen 1] [nodeEigenvector $nodeTag $eigen 2] [nodeEigenvector $nodeTag $eigen 3]]
        }
    }
}
# get ele tags
set eleTags [getEleTags]
foreach eleTag $eleTags {
    # get ele connections
    set eleNode [eleNodes $eleTag]
    if {[llength $eleNode] == 2} {
        set connections($eleTag) [format "\[%d, %d\]" [lindex $eleNode 0] [lindex $eleNode 1]]
    } elseif {[llength $eleNode] == 3} {
        set connections($eleTag) [format "\[%d, %d, %d\]" [lindex $eleNode 0] [lindex $eleNode 1] [lindex $eleNode 2]]
    } elseif {[llength $eleNode] == 4} {
        set connections($eleTag) [format "\[%d, %d, %d, %d\]" [lindex $eleNode 0] [lindex $eleNode 1] [lindex $eleNode 2] [lindex $eleNode 3]]
    }
}
# assemble
set yaml ""
append yaml "nodes:\n"
foreach nodeTag $nodeTags {
    append yaml "  $nodeTag:\n"
    append yaml [format "    coords: %s\n" $coordinates($nodeTag)]
    append yaml [format "    mass: %s\n" $masses($nodeTag)]
    if {$maxEigen == 0} {
        append yaml "    eigen: ~\n"
    } else {
        append yaml "    eigen:\n"
        foreach eigen $eigenYaml($nodeTag) {
            append yaml [format "      - %s\n" $eigen]
        }
    }
}
append yaml "elements:\n"
foreach eleTag $eleTags {
    append yaml "  $eleTag:\n"
    append yaml [format "    connection: %s\n" $connections($eleTag)]
}
return $yaml
}

################
#      3D      #
################
proc OpenSees2YAML3D {maxEigen} {
# eigen value solver
if {$maxEigen > 0} {
    puts "Eigen values:"
    puts [eigen $maxEigen]
} else {
    puts "Eigen value is not solved."
}
# get node tags
set nodeTags [getNodeTags]
foreach nodeTag $nodeTags {
    # node coordinates
    set coordinates($nodeTag) [format "\[%f, %f, %f\]" [nodeCoord $nodeTag 1] [nodeCoord $nodeTag 2] [nodeCoord $nodeTag 3]]
    # node mass
    set masses($nodeTag) [format "\[%f, %f, %f, %f, %f, %f\]" [nodeMass $nodeTag 1] [nodeMass $nodeTag 2] [nodeMass $nodeTag 3] [nodeMass $nodeTag 4] [nodeMass $nodeTag 5] [nodeMass $nodeTag 6]]
    # node eigen
    set eigenYaml($nodeTag) {}
    if {$maxEigen > 0} {
        for {set eigen 1} {$eigen <= $maxEigen} {incr eigen} {
            lappend eigenYaml($nodeTag) [format "\[%f, %f, %f, %f, %f, %f\]" [nodeEigenvector $nodeTag $eigen 1] [nodeEigenvector $nodeTag $eigen 2] \
                    [nodeEigenvector $nodeTag $eigen 3] [nodeEigenvector $nodeTag $eigen 4] [nodeEigenvector $nodeTag $eigen 5] [nodeEigenvector $nodeTag $eigen 6]]
        }
    }
}
# get ele tags
set eleTags [getEleTags]
foreach eleTag $eleTags {
    # get ele connections
    set eleNode [eleNodes $eleTag]
    if {[llength $eleNode] == 2} {
        set connections($eleTag) [format "\[%d, %d\]" [lindex $eleNode 0] [lindex $eleNode 1]]
    } elseif {[llength $eleNode] == 3} {
        set connections($eleTag) [format "\[%d, %d, %d\]" [lindex $eleNode 0] [lindex $eleNode 1] [lindex $eleNode 2]]
    } elseif {[llength $eleNode] == 4} {
        set connections($eleTag) [format "\[%d, %d, %d, %d\]" [lindex $eleNode 0] [lindex $eleNode 1] [lindex $eleNode 2] [lindex $eleNode 3]]
    }
}
# assemble
set yaml ""
append yaml "nodes:\n"
foreach nodeTag $nodeTags {
    append yaml "  $nodeTag:\n"
    append yaml [format "    coords: %s\n" $coordinates($nodeTag)]
    append yaml [format "    mass: %s\n" $masses($nodeTag)]
    if {$maxEigen == 0} {
        append yaml "    eigen: ~\n"
    } else {
        append yaml "    eigen:\n"
        foreach eigen $eigenYaml($nodeTag) {
            append yaml [format "      - %s\n" $eigen]
        }
    }
}
append yaml "elements:\n"
foreach eleTag $eleTags {
    append yaml "  $eleTag:\n"
    append yaml [format "    connection: %s\n" $connections($eleTag)]
}
return $yaml
}