####################
# filename: model.tcl
# author: Hanlin DONG
# description: A IDA model
# create: 2019-04-03 10:23:51
# license: MIT License (https://opensource.org/licenses/MIT)
# version: 1.0
####################

# Graph
#
#    0   1   2   3   4   5   6
# 3  o---o---o---o---o---o---o
#    | 1   1   1   1   1   1 | 1
# 2  o                       o
#    |                       | 1
# 1  o                       o
#    |                       | 1
# 0  *                       *

source modules/AxesTagging2D.tcl
source modules/OpenSees2YAML.tcl

proc portalFrame { RUN_ACCEL PGA } {
wipe
model BasicBuilder -ndm 2 -ndf 3
################
#  PARAMETERS  #
################
# Beam section H200 Height
set BEAM_H 200.
# Beam section H200 Width
set BEAM_W 200.
# Beam section H200 Thickness of Flange
set BEAM_TF 12.
# Beam section H200 Thickness of Web
set BEAM_TW 12.
# Beam material modulus of elasticity
set STEEL_E 2.0e6
# Beam material poission's ratio
set STEEL_NU 0.3
# Which acceleration timehistory to run
# set RUN_ACCEL 0
# Peak Ground Acceleration for the model (in g)
# set PGA 0.4
# Mass concentrated on beam column connection. Set as 15t.
set MASS_NODE 15.
###############
#    MODEL    #
###############
#####  TAGS  #####
set matRigid 1
set geomtransfSteel 1
##### MATERIAL #####
uniaxialMaterial Elastic $matRigid 1.0e12
##### GEOMETRIC TRANSFER #####
geomTransf Linear $geomtransfSteel
##### SECTION PROPERTIES ######
set STEEL_G [expr $STEEL_E / 2.0 / (1+$STEEL_NU)]
set ABeam [expr $BEAM_W * $BEAM_H - ($BEAM_W - $BEAM_TW) * ($BEAM_H - 2 * $BEAM_TF)]
set EBeam $STEEL_E
set IzBeam [expr ($BEAM_W * pow($BEAM_H, 3) - ($BEAM_W - $BEAM_TW) * pow($BEAM_H - 2 * $BEAM_TF, 3)) / 12.]
#####  GEOMETRY  #####
set XDists {1000. 1000. 1000. 1000. 1000. 1000}
set YDists {1000. 1000. 1000.}
set XCoords [AssembleCoordsList $XDists]
set YCoords [AssembleCoordsList $YDists]
##### CREATE NODES #####
CreateNodes 1 { 0 6 } { 0 1 2 3 } $XCoords $YCoords
CreateNodes 2 { 0 1 2 3 4 5 6 } { 3 } $XCoords $YCoords
# Fix column foots
FixNodes 1 { 0 6 } { 0 } { 1 1 1 }
# Set node mass
mass [GetNodeTag 1 0 3] $MASS_NODE $MASS_NODE 0 
mass [GetNodeTag 1 6 3] $MASS_NODE $MASS_NODE 0 
##### CREATE ELEMENTS #####
set tsColumn [CreateVerticalEles 1 { 0 6 } { 0 1 2 3 }]
set tsBeam [CreateHorizontalEles 2 { 0 1 2 3 4 5 6 } { 3 }]
foreach t [concat $tsColumn $tsBeam] {
    element elasticBeamColumn [lindex $t 0] [lindex $t 1] [lindex $t 2] $ABeam $STEEL_E $IzBeam $geomtransfSteel
}
##### BEAM COLUMN RIGID JOINT #####
set tsJoint [CreateZerolengthEles 1 1 2 { 0 6 } { 3 }]
foreach t $tsJoint {
    element twoNodeLink [lindex $t 0] [lindex $t 1] [lindex $t 2] -mat $matRigid $matRigid $matRigid -dir 1 2 3
}
##### LOG THE STRUCTURE #####
# set flog [open "structure.yml" "w"]
# puts $flog [OpenSees2YAML2D 1]
# close $flog
##### DEAD LOAD #####
set G 9800
timeSeries Linear 1 
pattern Plain 1 1 {
    load [GetNodeTag 1 0 3] 0 [expr -$MASS_NODE * $G] 0 
    load [GetNodeTag 1 6 3] 0 [expr -$MASS_NODE * $G] 0 
}
constraints Plain
numberer RCM
system BandGeneral
test EnergyIncr 1.0e-6 200
algorithm Newton
integrator LoadControl 1
analysis Static
analyze 1
##### Excitation ######
loadConst -time 0
# Peak Ground Acceleration of the accel.txt file.
set PGA_ACCEL { -0.2807955 -0.2807955 -0.2807955 }
timeSeries Path 2 -dt 0.01 -filePath source/accel$RUN_ACCEL.txt -factor [expr $PGA / [lindex $PGA_ACCEL $RUN_ACCEL]]
pattern UniformExcitation 2 1 -accel 2 -fact $G
constraints Transformation
numberer RCM
system UmfPack
test EnergyIncr 1.0e-6 200
algorithm Newton
integrator Newmark 0.5 0.25
analysis Transient
# get maximum lateral displacement
set maxDisp 0
for { set i 0 } { $i < 3000 } { incr i } {
    analyze 1 0.01
    set disp1 [nodeDisp [GetNodeTag 1 0 3] 1]
    set disp2 [nodeDisp [GetNodeTag 1 6 3] 2]
    if { $maxDisp < [expr abs($disp1)] } {
        set maxDisp $disp1
    }
    if { $maxDisp < [expr abs($disp2)] } {
        set maxDisp $disp2
    }
}
return $maxDisp
}

# Carry out the parametric study
set averageDisps {}
foreach pga { 0.1 0.2 0.3 } {
    set dispAll 0
    set dispNum 0
    foreach runAccel { 0 1 2 } {
        set disp [portalFrame $runAccel $pga]
        set dispAll [expr $dispAll + $disp]
        set dispNum [expr $dispNum + 1]
    }
    lappend averageDisps [expr $dispAll / $dispNum]
}
puts $averageDisps